@@ -22,7 +22,7 @@
 # along with SIDEKIT.  If not, see <http://www.gnu.org/licenses/>.
 
 """
-Copyright 2014-2020 Anthony Larcher
+Copyright 2014-2021 Anthony Larcher
 
 :mod:`frontend` provides methods to process an audio signal in order to extract
 useful parameters for speaker verification.
@@ -34,6 +34,7 @@ import logging
 import math
 import numpy
 import os
+import soundfile
 import struct
 import warnings
 import wave
@@ -45,7 +46,7 @@ from ..sidekit_wrappers import check_path_existance, process_parallel_lists
 
 
 __author__ = "Anthony Larcher"
-__copyright__ = "Copyright 2014-2020 Anthony Larcher"
+__copyright__ = "Copyright 2014-2021 Anthony Larcher"
 __license__ = "LGPL"
 __maintainer__ = "Anthony Larcher"
 __email__ = "anthony.larcher@univ-lemans.fr"
@@ -53,6 +54,8 @@ __status__ = "Production"
 __docformat__ = 'reStructuredText'
 
 
+wav_flag = "float32"    # Could be "int16"
+
 # HTK parameters
 WAVEFORM = 0
 LPC = 1
@@ -127,13 +130,18 @@ def read_wav(input_file_name):
     :param input_file_name:
     :return:
     """
-    with wave.open(input_file_name, "r") as wfh:
-        (nchannels, sampwidth, framerate, nframes, comptype, compname) = wfh.getparams()
-        raw = wfh.readframes(nframes * nchannels)
-        out = struct.unpack_from("%dh" % nframes * nchannels, raw)
-        sig = numpy.reshape(numpy.array(out), (-1, nchannels)).squeeze()
-        return sig.astype(numpy.float32), framerate, sampwidth
-    
+    #with wave.open(input_file_name, "r") as wfh:
+    #    (nchannels, sampwidth, framerate, nframes, comptype, compname) = wfh.getparams()
+    #    raw = wfh.readframes(nframes * nchannels)
+    #    out = struct.unpack_from("%dh" % nframes * nchannels, raw)
+    #    sig = numpy.reshape(numpy.array(out), (-1, nchannels)).squeeze()
+    #    return sig.astype(numpy.float32), framerate, sampwidth
+    nfo = soundfile.info(input_file_name)
+    sig, sample_rate = soundfile.read(input_file_name, dtype=wav_flag)
+    sig = numpy.reshape(numpy.array(sig), (-1, nfo.channels)).squeeze()
+    sig = sig.astype(numpy.float32)
+    return sig, sample_rate, 4
+
 
 def pcmu2lin(p, s=4004.189931):
     """Convert Mu-law PCM to linear X=(P,S)
@@ -460,7 +468,9 @@ def read_audio(input_file_name, framerate=None):
 def write_label(label,
                 output_file_name,
                 selected_label='speech',
-                frame_per_second=100):
+                frame_per_second=100,
+                show=None,
+                format="mdtm"):
     """Save labels in ALIZE format
 
     :param output_file_name: name of the file to write to
@@ -475,11 +485,29 @@ def write_label(label,
         # append 0 at the beginning of the list, append the last index to the list
         idx = [0] + (numpy.arange(len(bits))[bits] + 1).tolist() + [len(label)]
         framerate = decimal.Decimal(1) / decimal.Decimal(frame_per_second)
-        # for each pair of indexes (idx[i] and idx[i+1]), create a segment
-        with open(output_file_name, 'w') as fid:
+
+        if format == "lab":
+            # for each pair of indexes (idx[i] and idx[i+1]), create a segment
+            with open(output_file_name, 'w') as fid:
+                for i in range(~label[0], len(idx) - 1, 2):
+                    fid.write('{} {} {}\n'.format(str(idx[i]*framerate),
+                                                  str(idx[i + 1]*framerate), selected_label))
+        else:
+            # write in MDTM format
+            lst = []
             for i in range(~label[0], len(idx) - 1, 2):
-                fid.write('{} {} {}\n'.format(str(idx[i]*framerate),
-                                              str(idx[i + 1]*framerate), selected_label))
+                gender = 'U'
+                env = 'U'
+                channel = 'U'
+                start = idx[i]*framerate
+                stop = idx[i + 1]*framerate
+                lst.append('{:s} 1 {:.2f} {:.2f} {:s} {:s} {:s} {:s}\n'.format(
+                    show, start, stop - start, gender,
+                    channel, env, "speech"))
+
+            with open(output_file_name, 'w', encoding="utf8") as fid:
+                for line in lst:
+                    fid.write(line)
 
 
 def read_label(input_file_name, selected_label='speech', frame_per_second=100):
