@@ -2,6 +2,7 @@
 #
 # Copyright (c) 2019 Mirco Ravanelli
 # Copyright (c) 2019-2020 CNRS
+# Copyright (c) 2020-2021 LIUM
 #
 # Permission is hereby granted, free of charge, to any person obtaining a copy
 # of this software and associated documentation files (the "Software"), to deal
@@ -23,6 +24,7 @@
 #
 # AUTHOR
 # Hervé Bredin - http://herve.niderb.fr
+# Anthony Larcher
 
 # Part of this code was taken from https://github.com/mravanelli/SincNet
 # (see above license terms).
@@ -37,7 +39,6 @@
 from typing import List
 import numpy
 import torch
-#import torch.nn.functional as F
 import math
 
 
@@ -337,7 +338,6 @@ class SincNet(torch.nn.Module):
                     bias=True,
                 )
             else:
-                print(f"smaple_rate ={self.sample_rate} ")
                 conv1d = SincConv1d(
                                     out_channels,
                                     kernel_size,
