@@ -22,15 +22,15 @@
 # along with SIDEKIT.  If not, see <http://www.gnu.org/licenses/>.
 
 """
-Copyright 2014-2020 Anthony Larcher
+Copyright 2014-2021 Anthony Larcher
 
-The authors would like to thank the BUT Speech@FIT group (http://speech.fit.vutbr.cz) and Lukas BURGET
-for sharing the source code that strongly inspired this module. Thank you for your valuable contribution.
 """
 
+import glob
 import h5py
 import numpy
 import pandas
+import os
 import pickle
 import random
 import torch
@@ -50,9 +50,11 @@ from ..features_server import FeaturesServer
 from scipy.fftpack.realtransforms import dct
 from torchvision import transforms
 
+from torch.utils.data import DataLoader
+
 __license__ = "LGPL"
 __author__ = "Anthony Larcher"
-__copyright__ = "Copyright 2015-2020 Anthony Larcher"
+__copyright__ = "Copyright 2015-2021 Anthony Larcher"
 __maintainer__ = "Anthony Larcher"
 __email__ = "anthony.larcher@univ-lemans.fr"
 __status__ = "Production"
@@ -62,6 +64,32 @@ __docformat__ = 'reStructuredText'
 wav_type = "float32"  # can be int16, float64, int32 or float32
 
 
+def write_batch(batch_idx, data, label, batch_fn_format):
+    """
+
+    :param batch_idx:
+    :param data:
+    :param label:
+    :param batch_fn_format:
+    :return:
+    """
+    with h5py.File(batch_fn_format.format(batch_idx), "w") as h5f:
+        h5f.create_dataset('/data', data=data, fletcher32=True)
+        h5f.create_dataset('/label', data=label, fletcher32=True)
+
+def load_batch(batch_fn):
+    """
+
+    :param batch_fn:
+    :return:
+    """
+    with h5py.File(batch_fn, "r") as h5f:
+        data = h5f["/data"][()]
+        label = h5f["/label"][()]
+    return torch.from_numpy(data).type(torch.FloatTensor), torch.from_numpy(label).type(torch.LongTensor)
+
+
+
 def read_batch(batch_file):
     """
     :param batch_file:
@@ -80,6 +108,8 @@ def read_batch(batch_file):
         return data, label
 
 
+
+
 class XvectorDataset(Dataset):
     """
     Object that takes a list of files from a file and initialize a Dataset
@@ -338,7 +368,8 @@ class SideSet(Dataset):
                  chunk_per_segment=1,
                  overlap=0.,
                  dataset_df=None,
-                 min_duration=0.165
+                 min_duration=0.165,
+                 output_format="pytorch",
                  ):
         """
 
@@ -355,7 +386,7 @@ class SideSet(Dataset):
         self.data_file_extension = dataset["data_file_extension"]
         self.transformation = ''
         self.min_duration = min_duration
-
+        self.output_format = output_format
 
         if set_type == "train":
             self.duration = dataset["train"]["duration"]
@@ -503,7 +534,10 @@ class SideSet(Dataset):
                                                        self.add_reverb[index]
                                                        ))
 
-        return torch.from_numpy(sig).type(torch.FloatTensor), speaker_idx
+        if self.output_format == "pytorch":
+            return torch.from_numpy(sig).type(torch.FloatTensor), torch.from_numpy(speaker_idx).type(torch.LongTensor)
+        else:
+            return sig.astype(numpy.float32), speaker_idx
 
     def __len__(self):
         """
@@ -513,6 +547,29 @@ class SideSet(Dataset):
         """
         return self.len
 
+    def write_to_disk(self, batch_size, batch_fn_format, num_thread):
+        """
+
+        :param batch_size:
+        :param batch_fn_format:
+        :param num_thread:
+        :return:
+        """
+        # Check if the directory exists if not creates itbatch_fn_format
+        directory = os.path.dirname(batch_fn_format)
+        if not os.path.exists(directory):
+            os.makedirs(directory)
+
+        tmp_loader = DataLoader(self,
+                                batch_size=batch_size,
+                                shuffle=True,
+                                drop_last=True,
+                                pin_memory=True,
+                                num_workers=num_thread)
+
+        for batch_idx, (data, target) in tqdm.tqdm(enumerate(tmp_loader)):
+            write_batch(batch_idx, data, target, batch_fn_format)
+
 def createSideSets(data_set_yaml,
                    chunk_per_segment=1,
                    overlap=0.,
@@ -586,6 +643,13 @@ class IdMapSet(Dataset):
                     _transform.append(MFCC())
                 if "CMVN" in t:
                     _transform.append(CMVN())
+                if 'add_noise' in t:
+                    self.add_noise = numpy.ones(self.idmap.leftids.shape[0], dtype=bool)
+                    numpy.random.shuffle(self.add_noise)
+                    _transform.append(AddNoise(noise_db_csv="list/musan.csv",
+                                               snr_min_max=[5.0, 15.0],
+                                               noise_root_path="./data/musan/"))
+
         self.transforms = transforms.Compose(_transform)
 
     def __getitem__(self, index):
@@ -636,3 +700,35 @@ class IdMapSet(Dataset):
         :return:
         """
         return self.len
+
+
+class FileSet(Dataset):
+    """
+    Dataset class to load from disk
+    """
+
+    def __init__(self, batch_fn_format):
+        """
+
+        :param batch_fn_format:
+        """
+        self.batch_fn_format = batch_fn_format
+
+        # Get number of batches available on disk
+        batch_list = glob.glob(batch_fn_format.format('*'))
+        self.len = len(batch_list)
+
+    def __getitem__(self, idx):
+        """
+
+        :param idx:
+        :return:
+        """
+        return load_batch(self.batch_fn_format.format(idx))
+
+    def __len__(self):
+        """
+
+        :return:
+        """
+        return self.len
